package mydatastructures.datastructures;

public class LinkedListWithArrays {
	
	private int[] next;
	
	private int[] key;
	
	private int[] prev;
	
	private int head;
	
	private int capacity;
	
	private int free;
	
	public LinkedListWithArrays(int capacity) {
		next = new int[capacity];
		key = new int[capacity];
		prev = new int[capacity];
		this.capacity = capacity;
		
		free = 0;
		initNextArray();
		head = -1;
	}
	
	private void initNextArray() {
		for (int i = 0; i < capacity - 1; i++) {
			next[i] = i + 1;
		}
		next[capacity - 1] = -1;
	}
	
	public int getHead() {
		return key[head];
	}
	
	public void insert(int k) throws Exception {
		int index = allocateObject();
		key[index] = k;
		prev[index] = -1;
		next[index] = head;
		if (head >= 0) {
			prev[head] = index;
		}
		head = index;
	}
	
	private int search(int k) {
		for (int i = 0; i < key.length; i++) {
			if (k == key[i]) {
				return i;
			}
		}
		
		return -1;
	}
	
	public boolean isEmpty() {
		return head < 0;
	}
	
	public int getPrev(int k) {
		int index = search(k);
		return index >= 0 && prev[index] >= 0 ? key[prev[index]] : -1;		
	}
	
	public int getNext(int k) {
		int index = search(k);
		return index >= 0 && next[index] >= 0 ? key[next[index]] : -1;		
	}
	
	public void delete(int k) {
		int index = -1;
		if ((index = search(k)) >= 0) {
			if (prev[index] >= 0) {
				next[prev[index]] = next[index];
			} else {
				head = next[index];
			}
			
			if (next[index] >= 0) {
				prev[next[index]] = prev[index];
			}
			
			freeObject(index);
		}
	}
	
	private int allocateObject() throws Exception {
		if (free == -1) {
			throw new Exception("out of space");
		}
		
		int x = free;
		free = next[x];
		
		return x;
	}
	
	private void freeObject(int x) {
		next[x] = free;
		free = x;
	}

}
