package mygraphs.test;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertTrue;

import java.util.Arrays;

import mygraphs.algorithms.BFS;
import mygraphs.algorithms.DFS;
import mygraphs.repr.Graph;
import mygraphs.repr.Vertix;

import org.junit.Test;


public class GraphTest {

	@Test
	public void testGraphCreation() {
		Graph g = new Graph(new byte[][] { });
		assertTrue(g.getAdjacencyLists().isEmpty());
		
		g = new Graph( new byte[][] { {0} });
		assertTrue(g.getAdjacencyLists().get(new Vertix(0)).isEmpty());

		byte[][] matrix = new byte[][] { 
				{0, 1, 0, 0, 1},
				{1, 0, 1, 1, 1},
				{0, 1, 0, 1, 0},
				{0, 1, 1, 0, 1},
				{1, 1, 0, 1, 0}
		};
		
		g = new Graph(matrix);		
		assertEquals(Arrays.asList(new Vertix[] {new Vertix(1), new Vertix(4)}), g.getAdjacencyLists().get(new Vertix(0)));
		assertEquals(Arrays.asList(new Vertix[] {new Vertix(0), new Vertix(2), new Vertix(3), new Vertix(4)}), g.getAdjacencyLists().get(new Vertix(1)));
		assertEquals(Arrays.asList(new Vertix[] {new Vertix(1), new Vertix(3)}), g.getAdjacencyLists().get(new Vertix(2)));
		assertEquals(Arrays.asList(new Vertix[] {new Vertix(1), new Vertix(2), new Vertix(4)}), g.getAdjacencyLists().get(new Vertix(3)));
		assertEquals(Arrays.asList(new Vertix[] {new Vertix(0), new Vertix(1), new Vertix(3)}), g.getAdjacencyLists().get(new Vertix(4)));
	}
	
	@Test
	public void testBFS() {
		byte[][] matrix = new byte[][] { 
				{0, 1, 0, 0, 1},
				{1, 0, 1, 1, 1},
				{0, 1, 0, 1, 0},
				{0, 1, 1, 0, 1},
				{1, 1, 0, 1, 0}
		};
		
		Graph g = new Graph(matrix);
		
		BFS bfs = new BFS();	
		bfs.bfs(g, 0);
		
		assertEquals(1, g.getVertix(1).getDistance());
		assertEquals(1, g.getVertix(4).getDistance());
		
		assertEquals(2, g.getVertix(3).getDistance());
		assertEquals(2, g.getVertix(2).getDistance());
		
		assertEquals(0, g.getVertix(0).getDistance());
		
		g = new Graph(matrix);
		
		bfs.bfs(g, 1);
		
		assertEquals(1, g.getVertix(2).getDistance());
		assertEquals(1, g.getVertix(3).getDistance());
		assertEquals(1, g.getVertix(4).getDistance());
		
		assertEquals(0, g.getVertix(1).getDistance());
	}
	
	@Test
	public void testDFS() {
		byte[][] matrix = new byte[][] { 
				{0, 1, 0, 0, 1},
				{1, 0, 1, 1, 1},
				{0, 1, 0, 1, 0},
				{0, 1, 1, 0, 1},
				{1, 1, 0, 1, 0}
		};
		
		Graph g = new Graph(matrix);
		
		DFS dfs = new DFS();
		dfs.dfs(g);
		
		assertTrue(2 == g.getVertix(1).getDistance() || 2 == g.getVertix(1).getDistance());
		assertTrue(5 == g.getVertix(4).getDistance() || 3 == g.getVertix(4).getDistance());		
		assertTrue(4 == g.getVertix(3).getDistance() || 3 == g.getVertix(3).getDistance());
		assertTrue(3 == g.getVertix(2).getDistance() || 4 == g.getVertix(2).getDistance());
		
		assertEquals(1, g.getVertix(0).getDistance());		
	}

}
